/********************************************************************/
/*                                                                  */
/*   Program Name :  speana_vdif                                    */
/*                                                                  */
/*      Version:  1.00  2014-06-13                                  */
/*                                                                  */
/*      Copyright (c) 2014-     T.Kondo/NICT All Right Reserved     */
/*                                                                  */
/********************************************************************/
/*

  Spectrum analysis of VDIF data
        (off-line processing)

          T.Kondo 2002-05-20
               Ver 0.00 2014-04-16 start development based on speana 
               Ver 1.00 2014-06-13 
               Ver 1.10 2016-08-03 2bit decode mode default is changed to offset binary 


  How to run:

    speana_vdif vdif_file [options]  
    
       where  vdif_file -- vdif file name
                           if 0 is given, internal dflt name will be used

           options 
               -m[ode] YX 
			         axis scale mode
                       X: frequency axis mode
                          =0: log scale  =1: linear scale (default)
                       Y: amplitude axis mode
                          =0: log scale (default)  
                          =1: linear scale (common for 4 channels)
                          =2: linear scale (each max and min)
			   -a[uto_corr] [width]
			         set auto correlation mode.
					 width : set display width in time axis  
					         (deflt is full span)
               -i[nteg] integration_time
                     integration period (sec). Default is 1.0
               -p[lot_device] pmode
			         set plot device
                       0: XWINDOW and PostScript file out (default)
                       1: PostScript out only 
                       2: XWINDOW only 
               -c[omment] comment
			         comment (displayed at upper side of graph)
                     If omitted, you will be asked later.
               -s[offset] soffset
			         +VE : start offset (sec) (default is 0.0)
					 -VE : absolute value means skip over bytes 
               -k[ashima] [bytesframe] 
                     set header format to Kashima compulsory 
                     bytesframe: +VE bytes/frame set compulsory
			   -2bit mode     -- set 2bit AD decode table
			                         1: offset binary   0,1,2,3 (no conversion) (default)
									 2: Mark V          0,2,1,3
									 3: signed integer  1,0,2,3


*/

#define PROGRAM_VERSION "Ver. 2016-08-03"

//#define USE_GNUPLOT    // use GNUPLOT  (default is use of PGPLOT)
#include "libvdif.h"

#include "g_head.h"    // header for graphics

#ifdef WIN32
  #define USE_FFTW    // use FFTW package
#endif

#ifdef USE_FFTW
 #ifdef WIN32
  #include "fftw3.h"     // use of FFTW package
 #else
  #include <fftw3.h>     // use of FFTW package
 #endif
#endif

//#include "libk5.h"      // common header for IP-VLBI utilities

#include "libfft.h"

#ifdef WIN32
#define USE_ASM    // use inline assemble function
#else
#define USE_ASM    // use inline assemble function
#endif
  
//#include "libvssp32.h"

/*    Offset from dB to dBm conversion from A/D multibit sampling data */ 
#define OFFSET1  16.0     //1 bit sampling full power convert to 0 dB (theoretical)
      /* following constants are provisional  (measured on 2002/5/18) */
#define OFFSET2   9.0     //2 bit sampling data offset measured at 200kHz for PC(vlbi11)
#define OFFSET4  -1.0     //4 bit sampling data offset measured at 200kHz for PC(vlbi11)
#define OFFSET8 -26.0     //8 bit sampling data offset measured at 200kHz for PC(vlbi11)

#define BUFMAX 25000   // maximum read buffer size
//#define BUFMAX 2500   // maximum read buffer size

#define DFLT_DATA_FILE "vdif.dat"

#ifdef WIN32
   int tempcounter;  // counter for tempfile manage
#endif

#include "libvdif.h"
#include "libspeplot_v.h"


#define DEFLT_VDIF_FILE "vdif.dat"   // default vdif data file name

typedef struct speana_vdif_runp     // run parameters for vdifcheck
{
    char vdifname[256]; 
	char comment[60];
    int mode;   //  YX: axis scale mode
    double tinteg;
	double soffset;
	int64_t skip_bytes;   // skip bytes (1st skip over # of bytes)
    int pmode;
	int auto_flag;
	int smode;
	int flag_compulsory;
    int e2bit;
} speana_vdif_runp_t;



void print_usage(void)
{
    printf("speana_vdif %s\n",PROGRAM_VERSION);
    printf("\n");
    printf("Usage: speana_vdif vdif_file [options]\n");
    printf("\n");
    printf("       where  vdif_file -- vdif file name\n");
    printf("                           if 0 is given, internal dflt name will be used\n");
    printf("\n");
    printf("          options \n");
    printf("               -m[ode] YX \n");
    printf("                     axis scale mode\n");
    printf("                       X: frequency axis mode\n");
    printf("                          =0: log scale  =1: linear scale (default)\n");
    printf("                       Y: amplitude axis mode\n");
    printf("                          =0: log scale (default)  \n");
    printf("                          =1: linear scale (common for 4 channels)\n");
    printf("                          =2: linear scale (each max and min)\n");
    printf("               -a[uto_corr] [width]\n");
    printf("                     set auto correlation mode.\n");
    printf("                       width : set display width in time axis \n");
    printf("                               (deflt is full span)\n");
    printf("                     at the peak is displayed (magnified)\n");
    printf("               -i[nteg] integration_time\n");
    printf("                     integration period (sec). Default is 1.0\n");
    printf("               -p[lot_device] pmode\n");
    printf("                     set plot device\n");
    printf("                       0: XWINDOW and PostScript file out (default)\n");
    printf("                       1: PostScript out only \n");
    printf("                       2: XWINDOW only \n");
    printf("               -c[omment] comment\n");
    printf("                     comment (displayed at upper side of graph)\n");
    printf("                     If omitted, you will be asked later.\n");
    printf("               -s[offset] soffset\n");
    printf("                     +VE : start offset (sec) (default is 0.0)\n");
    printf("                     -VE : absolute value means skip over bytes \n");
    printf("               -k[ashima] [bytesframe]\n");
    printf("                     set header format to Kashima compulsory \n");
	printf("                     bytesframe: +VE is bytes/frame set compulsory\n");
    printf("               -2bit mode -- set 2bit AD decode table\n");
    printf("                           1: offset binary   0,1,2,3 (no conversion)(default)\n");
    printf("                           2: Mark V standard 0,2,1,3\n");
    printf("                           3: signed integer  1,0,2,3\n");
    printf("\n");
}

void param_get_speana_vdif(int argc, char *argv[], speana_vdif_runp_t *runp)                       
{
    char cbuf[256];

    // initialize parameters

    runp->mode=1;
	runp->auto_flag=0;   // auto correlation flag 
    runp->pmode=0;
    runp->vdifname[0]='\0';
	runp->comment[0]='\0';
	runp->tinteg=1.0;
	runp->soffset=0.0;
	runp->flag_compulsory=0;
	runp->skip_bytes=0;
    //runp->e2bit=2; // default
    runp->e2bit=1; // default  from 2016-08-03

    /* 1st parameter (vdif file name) get */
    if (argc>=2){
       strcpy(runp->vdifname,*(argv+1));
	   if (strlen(*(argv+1))==1 && runp->vdifname[0]=='0'){  // set default name
          strcpy(runp->vdifname,DEFLT_VDIF_FILE);
	   }
    }

    /* command parameters get */
    if(cmnd_get(argc,argv,"-m",cbuf)==0) runp->mode = atoi(cbuf);      // mode
    if(cmnd_get(argc,argv,"-i",cbuf)==0) runp->tinteg = atof(cbuf);    // integration
    if(cmnd_get(argc,argv,"-p",cbuf)==0) runp->pmode = atoi(cbuf);     // plot device
    if(cmnd_get(argc,argv,"-c",cbuf)==0) strcpy(runp->comment,cbuf);   // comment
	if(cmnd_get(argc,argv,"-s",cbuf)==0){
		if(atof(cbuf) >= 0.0){
		    runp->soffset = atof(cbuf);   // second offset
		} else {
			runp->skip_bytes=-atoi(cbuf);  // skip bytes
		}
	}
    if(cmnd_get(argc,argv,"-a",cbuf)==0){                              // auto correlation mode
		runp->auto_flag=1;
        if(cbuf[0] != '\0'){
            if(cbuf[0] != '-') runp->auto_flag=atoi(cbuf);
        }
    }
    if(cmnd_get(argc,argv,"-k",cbuf)==0 ||cmnd_get(argc,argv,"-K",cbuf)==0 ){ // set compulsory mode
		runp->flag_compulsory=1;
        if(cbuf[0] != '\0'){
            if(cbuf[0] != '-') runp->flag_compulsory=atoi(cbuf);
        }
    }
	if(cmnd_get(argc,argv,"-2bit",cbuf)==0) runp->e2bit = atoi(cbuf);   // 2bit encode mode

}



int main(int argc, char *argv[])
{
    int nloop;
    int i,n,m, k, kaisu;
    int w;
    int nfft;
    int ymode; // amplitude scale mode   0: log   1: linear
    int scount[16];  // allow max 16 channel
	//int upos,ss; // for debug
	int r_thread_id;
    double fact;
    double *cdata;
    double *f_hanning;
    double theta;
    double sum,dc;
    float pows;
    float pmax,pmin,pmaxdbm,pmindbm,cofst;
    float ymaxdb,ymindb;
#ifdef USE_FFTW
    fftw_complex *cc;
    fftw_plan pfcc, pbcc;
#endif
    double nfact;
    double tinteg;
    double telapsed;           // to measure processing time
    clock_t cstart1sec, cfinish1sec;  // to calculate processing time
    char ltime1st[60];
    char pgdevice_dflt[20];
    char* dflt_fname=DFLT_DATA_FILE;
    speana_vdif_runp_t runp;
	speanav_param_t st;
    vdif_file_t vd;
    timeval_vdif_t time1st;
    int *re_conv2bit;
    int re_conv2bit_1[]={0,1,2,3};   // offset binary (no conversion 2 bit)(default)
    int re_conv2bit_2[]={0,2,1,3};   //  conversion 2 bit
    int re_conv2bit_3[]={1,0,2,3};   //  conversion 2 bit singed integer


    /* set graphic device */
    strcpy(st.ps_fname,PS_OUT_FILE);
#ifdef USE_GNUPLOT   // using gnuplot
    strcpy(st.g_display,G_DISPLAY);
    strcpy(pgdevice_dflt,G_DISPLAY);
    strcpy(st.g_ps_device,G_PS_DEVICE);
#else
    strcpy(st.g_display,select_env_dflt(PGDISP, PGDISP_DFLT));
    strcpy(pgdevice_dflt,st.g_display);
    sprintf(st.g_ps_device,"%s%s",st.ps_fname,G_PS_DEVICE);
#endif

    if(argc == 1 || (argc == 2 && strncmp(*(argv+1),"--help",6)==0 )){ // 2008-02-19
          printf("speana_vdif %s",PROGRAM_VERSION);
#ifdef USE_FFTW
          printf(" compliled with FFTW3.0 \n");
#endif
#ifdef USE_GNUPLOT   /* using gnuplot */
          printf(" compliled for GNUPLOT \n");
#else
          printf(" compliled for PGPLOT \n");
#endif
          print_usage();

#ifndef USE_GNUPLOT   // using gnuplot
          printf("\n");
          printf("  Note: %s ---  environment variable for PGPLOT display device\n",PGDISP);
          printf("                   = %s",getenv(PGDISP));
          if(getenv(PGDISP)==NULL) {
              printf(",  program deflt is ( %s )\n",PGDISP_DFLT);
          } else {
              printf("\n");
          }
#endif
          printf("\n");
          exit(0);
    } 
    if(argc == 2 && strncmp(*(argv+1),"--ver",5)==0 ){
          printf("speana_vdif %s",PROGRAM_VERSION);
#ifdef USE_FFTW
          printf(" compliled with FFTW3.0 \n");
#endif
#ifdef USE_GNUPLOT   /* using gnuplot */
          printf(" compliled for GNUPLOT \n");
#else
          printf(" compliled for PGPLOT \n");
#endif
          printf("\n");
          exit(0);
    }

    param_get_speana_vdif(argc, argv, &runp); 

    if(runp.e2bit==1){
       re_conv2bit=re_conv2bit_1;
    } else if(runp.e2bit==3){
       re_conv2bit=re_conv2bit_3;
    } else {
       re_conv2bit=re_conv2bit_2;
    }

	strcpy(st.fname,runp.vdifname);
	strcpy(vd.fname,runp.vdifname);
	strcpy(st.comment,runp.comment);
	st.soffset=runp.soffset;

    // plot mode set
	st.mode2=runp.mode;
	st.mode=1;  // default frequency axis mode (1= linear scale)
	ymode=0;    // dfault amplitude axis mode (= log scale)
	st.amode=runp.auto_flag;  // auto correlation mode
//	if(st.mode2 <0 ){
//        st.amode=-st.mode2;
//        st.mode=0;
//    } else {
//        st.amode=0;
//        ymode=st.mode2/10;
//        st.mode=st.mode2 % 10;
//	}

	st.sekibun = runp.tinteg;
	st.pmode = runp.pmode;

    printf("speana_vdif %s",PROGRAM_VERSION);
#ifdef USE_FFTW
    printf(" compliled with FFTW3.0 \n");
#else
    printf("\n");
#endif

    cstart1sec = clock();

    /* get comment */

//	if (st.comment[0]=='\0'){
//       /* enter any comment */
//       printf("Enter any comment -> ");
//       read1line(st.comment);      // 1 line read from stdin
//    }

	vd.skip_bytes=runp.skip_bytes;
	vd.flag_compulsory=runp.flag_compulsory;

    if(vdif_t_ini(&vd,0)<0) goto exitend2;   // VDIF data check with auto mode
    //if(vdif_t_ini(&vd,1)<0) goto exitend2;  // VDIF data check with 1 sec reading compulsory

	time1st=vd.vhead.t;  // save 1st time 

    /* parameter set */
	st.frames_per_sec=vd.nframsec;
	st.adbit=vd.adbit;   // AD bits
	st.sfreq=vd.fsample; 
	st.numch=vd.numch;
	
	/* 
	  set reading buffer size to satisfy following condition
	     1. (frame size)/(buffer size) should be an integer number
	*/
	vd.nword=vd.frame_bytes/4;     
	while (vd.nword > BUFMAX) vd.nword/=2;

	if(((vd.frame_bytes/4) % vd.nword) != 0){
		printf(" speana_vdif: buffer size does\'nt satisfy the condition! %d %d\n",vd.nword, vd.frame_bytes*4);
	}

	vd.imax=(vd.frame_bytes/4)/vd.nword;   // # ofloops to read entire one frame data
	vd.nsample=vd.nword*32/vd.adbit;  // # of samples in buffer (don't care about # of Chs)

	vd.imax_c=0;

	runp.smode=1;   // mode to choose unit processing sample numbers per channel
	vd.smode=runp.smode;
	/* set processing unit number of samples per channel */
    if(vd.smode ==0){ vd.usampl=200000; }
    else if (vd.smode ==1) {
       vd.usampl=10000;
    } else {
       vd.usampl=1000;      // for smode=2
    }
    //    check usampl condition 
    //    because usampl*adbit*numch/32 must be an integer (<= this comes from K5 fx_cor)
	//    if data is not used for K5 fx_cor, this condition can be ignored.
    if((vd.usampl*vd.adbit*vd.numch)%32 != 0) {
        while ((vd.usampl*vd.adbit*vd.numch)%32 != 0) {
             vd.usampl*=2;
        }
		printf(" new usampl %d\n",vd.usampl);
    }

	//if(vd.usampl*vd.numch > vd.nsample) vd.usampl=vd.nsample/vd.numch;

	vd.unitp=(double)vd.usampl/vd.fsample;   // period of usampl


	//printf("imax nword32  %d %d\n",vd.imax,vd.nword);    //debug

    nfft=1024;
    if(vd.usampl < 524288) nfft= 524288;
    if(vd.usampl < 262144) nfft= 262144;
    if(vd.usampl < 131072) nfft= 131072;
    if(vd.usampl < 65536)  nfft= 65536;
    if(vd.usampl < 32768)  nfft= 32768;
    if(vd.usampl < 16384)  nfft= 16384;

    st.nfft=nfft;

    nfact=2.0/(double)nfft;
    vd.ubuf  = (u_int32_t *)malloc(vd.nword*sizeof(u_int32_t)); //  data read buffer
	//vd.idata  = (int *)malloc((vd.usampl+32)*vd.numch*sizeof(int)); //  data pick up buffer (32 is overflow space)
    vd.ch   = (ch_data_t *)malloc(vd.numch*sizeof(ch_data_t));
    for(n=0; n<vd.numch; n++)
    {
        vd.ch[n].idata=(int *)malloc((vd.usampl+32)*vd.numch*sizeof(int)); //  data pick up buffer (32 is overflow space)
    }

#ifdef USE_FFTW
    cc=fftw_malloc(sizeof(fftw_complex)*nfft);

    pfcc=fftw_plan_dft_1d(nfft, cc, cc, FFTW_FORWARD, FFTW_ESTIMATE);
    pbcc=fftw_plan_dft_1d(nfft, cc, cc, FFTW_BACKWARD, FFTW_ESTIMATE);
#endif

    cdata = (double *)malloc(2*nfft*sizeof(double));
    //st.spows = (float *)malloc(4*nfft*sizeof(float));
    st.spows = (float *)malloc(vd.numch*nfft*sizeof(float));
    f_hanning = (double *)malloc(nfft*sizeof(double));

    //Hanning filter coeeficients prepare
    for(k=0; k<nfft; k++){
        theta=360.0*(double)k/(double)(nfft-1) - 180.0;
        theta=fmod(theta,360.0) ;
        f_hanning[k]=0.5*(1.0+cos(theta*PIRA));
    }

    fact=((float)vd.usampl)*((float)vd.usampl);


    for(k=0; k<vd.numch*nfft; k++) st.spows[k]=0.0;
    for(k=0; k<vd.numch; k++) scount[k]=0;

	// file open
	if( (vd.fin  = fopen( vd.fname, "rb" )) == NULL ){
         printf("File (%s) Open Error! \n",vd.fname);
         goto exitend2;
    }
	vd.flag_open=1;

	if(vd.flag_vdif_anal==1){ // read pointer move to 1st valid header
		fseek(vd.fin,vd.pos_1st_header,SEEK_SET); // move pointer by K5->bytesec
	}

    /* foward startoffset seconds */
	if(st.soffset>0){
		printf("  %d second forwading.....",(int)st.soffset);
	    if(fwdNsecVDIF((int)st.soffset, &vd) <0){
           printf("Foward error %d\n",(int)st.soffset);
           i=fclose( vd.fin ) ;
           goto exitend;
        }
	    printf("\n");
	    /* get headder info again */
        if(vdif_header_read_mode1(&vd)<0) goto exitend;
	    time1st=vd.vhead.t;
	}

	sprintf(ltime1st,"Time(UT) %04d/%02d/%02d %02d:%02d:%02d",
		time1st.year,time1st.month,time1st.mday,time1st.hh,time1st.mm,time1st.ss);
    //sprintf(ltime1st,"Debugd");  // debug
    strcpy(st.lab2,ltime1st);
    strcat(st.lab2,"  ");
    strcat(st.lab2,st.comment);

	// vdif read 

	r_thread_id=vd.vhead.thread_id;  // set reading thread ID to 1st one
        
    tinteg=0.0;
	vd.pos=0;

	//st.numch=1; // assuming 1 ch data

    nloop=0;

	vd.tot_frame=0; // total frame counter
	vd.upos=0;
	vd.usampl_c=0;
	vd.n_usampl=0;
	vd.flag_read_header=1;
	vd.flag_read_data=1;
	vd.flag_idata_over=0;

    printf("  Reading thread ID is  %d\n",r_thread_id);
    printf("  Data reading.....\n");


loop1:

	    //upos=vd.upos;    // debug
		//ss=vd.vhead.t.ss; // debug

		i=vdif_read_unit(&vd,r_thread_id);

		//printf("%02d:%02d:%02d %lld %f\n",vd.vhead.t.hh,vd.vhead.t.mm,vd.vhead.t.ss,vd.tot_frame,vd.dtime); //debug

		//debug
		//if(vd.vhead.t.ss != ss){  // debug
		//  printf("%02d:%02d:%02d %lld %f\n",vd.vhead.t.hh,vd.vhead.t.mm,vd.vhead.t.ss,vd.tot_frame,vd.dtime); //debug
		//	printf("upos %d  %d  \n",upos,vd.upos);
		//}

		if(i==-99){ // EOF
			printf("\n");
			goto exitloop; 
		} else {
			if(i < 0){
			  printf("\n");
			  goto exitend; 
			}
		}

		for(m=0; m<vd.numch; m++){
		    //m=0;
            kaisu=0;
            sum=0.0;

            // initialize array
#ifdef USE_FFTW
            for(k=0; k<nfft; k++){ cc[k][0]=0.0; cc[k][1]=0.0;}
#else
            for(k=0; k<2*nfft; k++) cdata[k]=0.0;
#endif

			for(k=0; k<vd.npos; k++){

                /* set data */
                if(st.adbit==2){
                    w=re_conv2bit[vd.ch[m].idata[k]];
                } else {
                    w=vd.ch[m].idata[k];
                }
#ifdef USE_FFTW
                cc[k][0]=(double)w;  
                sum+=cc[k][0];
#else
                cdata[2*k]=(double)w;  
                sum+=cdata[2*k];
#endif
                kaisu++;
            }
             
            dc=sum/(double)kaisu;    // DC component
     
            // DC offset removal from data
            for(k=0; k<vd.npos; k++){
            //for(k=0; k<vd.npos; k++){
#ifdef USE_FFTW
                cc[k][0]-=dc;
#else
                cdata[2*k]-=dc;
#endif
            }

            if(st.amode ==0) {    // spectrum mode
                for(k=0; k<nfft; k++){
#ifdef USE_FFTW
                   cc[k][0]*=f_hanning[k];  // hanning filter
                   cc[k][1]*=f_hanning[k];  // hanning filter
#else
                   cdata[2*k]*=f_hanning[k];
                   cdata[2*k+1]*=f_hanning[k];
#endif
                }
                //hanning(cdata,usampl);  // hanning filter
#ifdef USE_FFTW
                fftw_execute(pbcc);
#else
                cdft(2*nfft,1,cdata);
#endif
            } else {
#ifdef USE_FFTW
                fftw_execute(pfcc);
                for(k=0; k<nfft; k++){
                    cc[k][0]*=nfact; cc[k][1]*=nfact;
                }
#else
                cdft(2*nfft,-1,cdata);
#endif
            }
            for(k=0; k<nfft; k++){
#ifdef USE_FFTW
                pows=(float)((cc[k][0]*cc[k][0]+cc[k][1]*cc[k][1])/(double)fact);
#else
                pows=(float)((cdata[2*k]*cdata[2*k]+cdata[2*k+1]*cdata[2*k+1])/(double)fact);
#endif
                st.spows[k*vd.numch+m]=st.spows[k*vd.numch+m]+pows;
            }

            scount[m]++;
        }
		tinteg+=vd.unitp;
        if(tinteg >= st.sekibun) goto exitloop;

        goto loop1;

exitloop:
    if(vd.flag_open==1){
		i=fclose( vd.fin ) ;
		vd.flag_open=0;
	}

	printf("\n");

    if(st.amode > 0) {   // auto correlation function calc
       for(m=0; m<vd.numch; m++){
          for(k=0; k<nfft; k++){
             st.spows[k*vd.numch+m]/=scount[m]; // nomalization
#ifdef USE_FFTW
             cc[k][0]=st.spows[k*vd.numch+m]; cc[k][1]=0.0;
#else
             cdata[2*k]=st.spows[k*vd.numch+m];
             cdata[2*k+1]=0.0;
#endif
          }
          if(m==1 || m==3){
#ifdef USE_FFTW
              cc[0][0]=1.0;
#else
              cdata[0]=1.0;  // to avoid math error
#endif
          }
#ifdef USE_FFTW
          fftw_execute(pbcc);
#else
          cdft(2*nfft,1,cdata);
#endif
          for(k=0; k<nfft; k++){
             // if(k<10) printf("%d %e\n",k,cdata[2*k]);  // debug
             // if(k>nfft-10) printf("%d %e\n",k,cdata[2*k]);  // debug
#ifdef USE_FFTW
             if(k==0) pmax=(float)cc[0][0];
             if(cc[k][0] > pmax) pmax=(float)cc[k][0];
             st.spows[k*vd.numch+m]=(float)cc[k][0];
#else
             if(k==0) pmax=(float)cdata[0];
             if(cdata[2*k] > pmax) pmax=(float)cdata[2*k];
             st.spows[k*vd.numch+m]=(float)cdata[2*k];
#endif
          }
          for(k=0; k<nfft; k++){  // normalize
             st.spows[k*vd.numch+m]/=pmax;
             // if(k<10) printf("   %d %e\n",k,spows[k*4+m]);  // debug
             // if(k>nfft-10) printf("   %d %e\n",k,spows[k*4+m]);  // debug
          }
             
       }
         
    } else {

        /* convert power to dB  then dBm */       
        cofst=OFFSET1;       //1 bit sampling full power convert to 0 dB
        if( st.adbit==8) cofst=OFFSET8;     // 8bit sampling offset
        if( st.adbit==4) cofst=OFFSET4;     // 4bit sampling offset
        if( st.adbit==2) cofst=OFFSET2;     // 4bit sampling offset
        for(m=0; m<st.numch; m++){
            for(k=0; k<nfft; k++){
                st.spows[k*st.numch+m]/=scount[m]; // nomalization
                st.spows[k*st.numch+m]=(float)(10.0*log10(st.spows[k*st.numch+m]) + cofst); 
                if(k==0){
                    pmaxdbm=st.spows[k*st.numch+m];
                    pmindbm=st.spows[k*st.numch+m];
                    if(m==0){
                        ymaxdb=pmaxdbm;
                        ymindb=pmindbm;
                    }
                } else {
                    if(st.spows[k*st.numch+m] > pmaxdbm) pmaxdbm=st.spows[k*st.numch+m]; 
                    if(st.spows[k*st.numch+m] < pmindbm) pmindbm=st.spows[k*st.numch+m]; 
                }
            }
            pmax=pmaxdbm-cofst;
            pmin=pmindbm-cofst;
            printf("CH# %d: Maximum data (dB, dBm) is %f %f\n", m+1,pmax,pmaxdbm);      
            if(pmaxdbm > ymaxdb) ymaxdb=pmaxdbm;
            if(pmindbm < ymindb) ymindb=pmindbm;
        }
        printf("All CH: Maximum and Minimum data (dBm) are %f %f\n", ymaxdb, ymindb);      

    }
    cfinish1sec = clock();
    telapsed = (double)(cfinish1sec - cstart1sec) / CLOCKS_PER_SEC;
    printf("Time elapsed for processing is %f sec\n",telapsed);

    /* plot spectrum (or auto cor) POSTSCRIPT OUT and XTERM*/
     plotspe_v(&st,ymaxdb, ymindb);

  exitend: ;
    if(vd.flag_open==1){
		i=fclose( vd.fin ) ;
		vd.flag_open=0;
	}
      
    free(vd.ubuf);
    //free(vd.idata);
    for(n=0; n<vd.numch; n++) free(vd.ch[n].idata);
    free(vd.ch);
    free(cdata);
    free(st.spows);

#ifdef USE_FFTW
    fftw_destroy_plan(pfcc);
    fftw_destroy_plan(pbcc);
    fftw_free(cc);
#endif

 exitend2: ;

    return 0;
}
